#!/usr/bin/python

from sys        import argv, stderr
from os         import remove, devnull
from subprocess import call, check_output
from time       import time
from psts       import genpsts
from coco       import coconad
from coconad    import coconad as pycoconad

#-----------------------------------------------------------------------

def run_pipe (prog, null):
    res = []
    for line in check_output(prog +['-'],stderr=null).split('\n')[:-1]:
        r = [int(x) for x in line.split()]
        res.append((r[:-1], r[-1]))
    return res

#-----------------------------------------------------------------------

def run_file (prog, null):
    call(prog +['tmp.txt'], stderr=null)
    res = []
    with open('tmp.txt', 'rt') as inp:
        for line in inp:
            r = [int(x) for x in line.split()]
            res.append((r[:-1], r[-1]))
    remove('tmp.txt')
    return res

#-----------------------------------------------------------------------

if __name__ == '__main__':
    runs = int(argv[1]) if len(argv) > 1 else 1
    ok   = True

    stderr.write('generating data ... '); t = time()
    psts = zip(range(1,101), genpsts(100, 3, 20))
    with open('data.txt', 'w') as out:
        for n,x in psts:
            for s in x: out.write('%d %g\n' % (n,s))
    psts = dict()
    with open('data.txt', 'r') as inp:
        for line in inp:
            n,s = line.split()
            n,s = int(n),float(s)
            if n in psts: psts[n].append(s)
            else:         psts[n] = [s]
    psts = psts.items()
    # The generated data needs to be re-read to compensate
    # rounding errors resulting from the writing.

    stderr.write('repetitions: %d\n' % runs)

    stderr.write('python    ... '); t = time()
    for r in range(runs):
        pypats = pycoconad(psts, width=0.003, supp=2, zmin=2)
    stderr.write('done [%d, %.3fs].\n' % (len(pypats),time()-t))
    pypats = set([(tuple(sorted(list(s))), x) for s,x in pypats])

    stderr.write('coconad f ... '); t = time()
    for r in range(runs):
        c_pats = coconad  (psts, width=0.003, supp=2, zmin=2, algo='f')
    stderr.write('done [%d, %.3fs].\n' % (len(c_pats),time()-t))
    c_pats = set([(tuple(sorted(list(s))), x) for s,x in c_pats])

    if pypats == c_pats: print('ok')
    else: ok = False;    print('fail')

    stderr.write('coconad r ... '); t = time()
    for r in range(runs):
        c_pats = coconad  (psts, width=0.003, supp=2, zmin=2, algo='r')
    stderr.write('done [%d, %.3fs].\n' % (len(c_pats),time()-t))
    c_pats = set([(tuple(sorted(list(s))), x) for s,x in c_pats])

    if pypats == c_pats: print('ok')
    else: ok = False;    print('fail')

    prg = ['coconad', '-w0.003', '-s2', '-m2', '-v %a', 'data.txt']
    stderr.write('pipe      ... '); t = time()
    with open(devnull, 'w') as null:
        for r in range(runs): expats = run_pipe(prg, null)
    stderr.write('done [%d, %.3fs].\n' % (len(expats),time()-t))

    stderr.write('file      ... '); t = time()
    with open(devnull, 'w') as null:
        for r in range(runs): expats = run_file(prg, null)
    stderr.write('done [%d, %.3fs].\n' % (len(expats),time()-t))
    expats = set([(tuple(sorted(list(s))), x) for s,x in expats])

    if pypats == expats: print('ok')
    else: ok = False;    print('fail')

    if ok: remove('data.txt')
