#!/usr/bin/python3

from setuptools import setup, Extension
from os         import remove

version='2.44'
description='COntinuous-time ClOsed Neuron Assembly Detection (2016.11.21)'

dirs   =['util/src',
         'math/src',
         'tract/src',
         'coconad/src',
         'pycoco/src']
headers=['util/src/arrays.h',
         'util/src/memsys.h',
         'util/src/symtab.h',
         'util/src/random.h',
         'util/src/sigint.h',
         'util/src/fntypes.h',
         'math/src/gamma.h',
         'tract/src/tract.h',
         'tract/src/train.h',
         'tract/src/patspec.h',
         'tract/src/clomax.h',
         'tract/src/cm4seqs.h',
         'tract/src/cmfilter.h',
         'tract/src/report.h',
         'tract/src/patred.h',
         'coconad/src/coconad.h',
         'coconad/src/ccnpsp.h']
sources=['util/src/arrays.c',
         'util/src/memsys.c',
         'util/src/symtab.c',
         'util/src/random.c',
         'util/src/sigint.c',
         'math/src/gamma.c',
         'tract/src/tract.c',
         'tract/src/train.c',
         'tract/src/patspec.c',
         'tract/src/clomax.c',
         'tract/src/cm4seqs.c',
         'tract/src/cmfilter.c',
         'tract/src/report.c',
         'tract/src/patred.c',
         'coconad/src/coconad.c',
         'coconad/src/ccnpsp.c',
         'pycoco/src/pycoco.c']
macros =[('NDEBUG',      None),
         ('IDMAPFN',     None),
         ('TRN_SURR',    None),
         ('PSP_ESTIM',   None),
         ('PSP_TRAIN',   None),
         ('ISR_PATSPEC', None),
         ('ISR_CLOMAX',  None),
         ('ISR_NONAMES', None),
         ('CCN_ABORT',   None)]

with open('MANIFEST.in', 'wt') as out:
    for h in headers: out.write('include '+h+'\n')
with open('README', 'wt') as out:
    out.write('Call help() on the functions\n')
    out.write('  coconad\n')
    out.write('  genpsp\n')
    out.write('  estpsp\n')
    out.write('  psp2bdr\n')
    out.write('  patred\n')
    out.write('for explanations about their parameters.\n')

setup(name='coco',
      version=version,
      description=description,
      long_description=description,
      author='Christian Borgelt',
      author_email='christian@borgelt.net',
      maintainer='Christian Borgelt',
      maintainer_email='christian@borgelt.net',
      url='http://www.borgelt.net/pycoco.html',
      download_url='http://www.borgelt.net/pycoco.html',
      platforms='Linux/Unix,Microsoft Windows',
      license='MIT License (Expat License)',
      ext_modules=[Extension('coco', sources, include_dirs=dirs,
                             define_macros=macros)])

remove('MANIFEST.in')
remove('README')
